--------------------------------------------------------------------------------------
-- Engine Script for Class 47
--
-- (c) Railsimulator.com 2011
--
-- Provides script for variable exhaust smoke.
-- Animates cooling vents based on RPM.
--
--------------------------------------------------------------------------------------


-- maximum tractive effort for determining smoke density. Acceleration rate for AI.

MAXTE = 0.5
ACCELRATE = 0.25

gSmokeColourR = 1.0
gSmokeColourG = 1.0
gSmokeColourB = 1.0
gEmitRate = 0.05

-------------------------------------------------------------------------------------

function Initialise ()


-- Get emitter values.

	gSmokeColourR, gSmokeColourG, gSmokeColourB = Call( "Exhaust:GetEmitterColour" )


-- Stores for checking when values have changed.

	gDriven = 0

-- Stores for vent positions.

	gFrontVent = 0
	gBackVent = 0


	Call( "BeginUpdate" )
	
end

-------------------------------------------------------------------------------------

function OnControlValueChange ( name, index, value )

	if Call( "*:ControlExists", name, index ) then

		Call( "*:SetControlValue", name, index, value );

	end

end

-------------------------------------------------------------------------------------

function Update ( time_inc )


-- Check for player train.

	if Call( "GetIsPlayer" ) == 1 then

-- Check if player is driving this engine.

		if ( Call( "GetIsEngineWithKey" ) == 1 ) then
			if gDriven == 0 then
				gDriven = 1
				Call( "*:SetControlValue", "Active", 0, 1 )
			end
		else
			if gDriven == 1 then
				gDriven = 0
				Call( "*:SetControlValue", "Active", 0, 0 )
			end
		end
		
		-- control de amperaje en motores de traccion
		
		if gDriven == 1 then
		amp=Call( "*:GetControlValue", "Ammeter", 0 )
		if amp > 3000 then
		Call ( "*:SetControlValue", "EmergencyBrake", 0, 1)
		--Call ( "*:SetControlValue", "EngineStop", 0, 0)
		Call ( "*:SetControlValue", "Startup", 0, -1)
		--Call ( "*:SetControlValue", "RPM", 0, 0)
		end
		end

-- Get tractive effort values for this vehicle.
		TractiveEffort = Call ( "GetTractiveEffort" )
		TractiveEffort = math.abs ( TractiveEffort )

	else -- This is an AI train.

-- Tractive effort doesn't work for AI, so use acceleration.

		Accel = Call( "GetAcceleration" )
		Speed = Call( "GetSpeed" )

-- Unfortunately acceleration is signed based on direction loco is facing (sic).
		if Speed < 0 then
			Accel = Accel * -1
		end

		if Accel > ACCELRATE then
			TractiveEffort = MAXTE
		elseif ( math.abs(Speed) > 1 ) and ( Accel > -0.1 ) then
			TractiveEffort = MAXTE/2
		else
			TractiveEffort = MAXTE/11
		end

	end


-- Get rpm value for this vehicle.
	rpm = Call( "*:GetControlValue", "RPM", 0 )


-- Adjust exhaust smoke based on tractive effort.

	if TractiveEffort < MAXTE/10 then
						
		gSmokeColourR = 0.8
		gSmokeColourG = 0.8
		gSmokeColourB = 0.8
		Call ( "Exhaust:SetEmitterColour", gSmokeColourR, gSmokeColourG, gSmokeColourB )
		Call ( "Exhaust:SetEmitterRate", gEmitRate )

	end
				
	if ( TractiveEffort <= MAXTE/2 ) and ( TractiveEffort >= MAXTE/10 ) then
				
		gSmokeColourR = 0.25
		gSmokeColourG = 0.25
		gSmokeColourB = 0.25
		Call ( "Exhaust:SetEmitterColour", gSmokeColourR, gSmokeColourG, gSmokeColourB )
		Call ( "Exhaust:SetEmitterRate", 0.01 )

	end
		
	if TractiveEffort > MAXTE/2 then 
	
		gSmokeColourR = 0.0
		gSmokeColourG = 0.0
		gSmokeColourB = 0.0
		Call ( "Exhaust:SetEmitterColour", gSmokeColourR, gSmokeColourG, gSmokeColourB )
		Call ( "Exhaust:SetEmitterRate", 0.005 )

	end


-- Vent animations.

	if rpm < 220 then
		if gFrontVent == 1 then
			if ( Call( "*:AddTime", "FrontVents", -time_inc ) ~= 0 ) then
				gFrontVent = 0
			end
		end
		if gBackVent == 1 then
			if ( Call( "*:AddTime", "BackVents", -time_inc ) ~= 0 ) then
				gBackVent = 0
			end
		end
	elseif ( rpm >= 220 ) and ( rpm <= 600 ) then
		if gFrontVent == 0 then
			if ( Call( "*:AddTime", "FrontVents", time_inc ) ~= 0 ) then
				gFrontVent = 1
			end
		end
		if gBackVent == 1 then
			if ( Call( "*:AddTime", "BackVents", -time_inc ) ~= 0 ) then
				gBackVent = 0
			end
		end
	else -- RPM must be greater than 600
		if gFrontVent == 0 then
			if ( Call( "*:AddTime", "FrontVents", time_inc ) ~= 0 ) then
				gFrontVent = 1
			end
		end
		if gBackVent == 0 then
			if ( Call( "*:AddTime", "BackVents", time_inc ) ~= 0 ) then
				gBackVent = 1
			end
		end
	end

end


